#ifndef PKMNLIB_BATTLELIBRARY_HPP
#define PKMNLIB_BATTLELIBRARY_HPP

#include <CreatureLib/Battling/Library/BattleLibrary.hpp>
#include "../../Library/PokemonLibrary.hpp"
#include "CaptureLibrary.hpp"
#include "DamageLibrary.hpp"
#include "ExperienceLibrary.hpp"
#include "MiscLibrary.hpp"
#include "ScriptResolver.hpp"
#include "StatCalculator.hpp"

namespace PkmnLib::Battling {
    class BattleLibrary final : public CreatureLib::Battling::BattleLibrary {
        std::unique_ptr<const CaptureLibrary> _captureLibrary;

    public:
        BattleLibrary(Library::PokemonLibrary* non_null staticLib, StatCalculator* non_null statCalculator,
                      DamageLibrary* non_null damageLibrary, ExperienceLibrary* non_null experienceLibrary,
                      ScriptResolver* non_null scriptResolver, MiscLibrary* non_null miscLibrary,
                      CaptureLibrary* non_null captureLibrary)
            : CreatureLib::Battling::BattleLibrary(staticLib, statCalculator, damageLibrary, experienceLibrary,
                                                   scriptResolver, miscLibrary),
              _captureLibrary(captureLibrary) {}

        const std::unique_ptr<const Library::LibrarySettings>& GetSettings() const {
            return reinterpret_cast<const std::unique_ptr<const Library::LibrarySettings>&>(_staticLib->GetSettings());
        }

        const std::unique_ptr<const Library::PokemonLibrary>& GetStaticLib() const {
            return reinterpret_cast<const std::unique_ptr<const Library::PokemonLibrary>&>(_staticLib);
        }

        const std::unique_ptr<const Library::SpeciesLibrary>& GetSpeciesLibrary() const {
            return reinterpret_cast<const std::unique_ptr<const Library::SpeciesLibrary>&>(
                _staticLib->GetSpeciesLibrary());
        }

        const std::unique_ptr<const Library::MoveLibrary>& GetMoveLibrary() const {
            return reinterpret_cast<const std::unique_ptr<const Library::MoveLibrary>&>(_staticLib->GetAttackLibrary());
        }

        const std::unique_ptr<const Library::ItemLibrary>& GetItemLibrary() const {
            return reinterpret_cast<const std::unique_ptr<const Library::ItemLibrary>&>(_staticLib->GetItemLibrary());
        }

        const ArbUt::BorrowedPtr<const Library::NatureLibrary> GetNatureLibrary() const {
            return GetStaticLib()->GetNatureLibrary();
        }
        [[nodiscard]] inline const ArbUt::BorrowedPtr<const CaptureLibrary> GetCaptureLibrary() const noexcept {
            return _captureLibrary;
        }

        static PkmnLib::Battling::ScriptResolver* non_null CreateScriptResolver();
    };
}
#endif // PKMNLIB_BATTLELIBRARY_HPP
